#!/usr/bin/perl

#this script is used for auto GlimmerHMM
#split the genome into single seq files
#Modify this file if you want to change the way it works
#this script runs with glimmerhmm-linux, so please specify the dir where it is.
#author: gaoshh@big.ac.cn
#v0.1 Nov 18, 2010 basic wrapper for GlimmerHMM
#v0.2 Feb 10, 2012 multi-threaded 

#modules
use Getopt::Long;
use Bio::Seq;
use Bio::SeqIO;
use File::Spec::Functions qw/rel2abs/;
use File::Basename;
use Thread;

#get options
use Getopt::Long;
my %opts;
GetOptions(\%opts,"g:s","p:s","d:s","w:s","o:s","n:n","gff","v","f","t:n","h");

if ((!$opts{g})||(!$opts{d})||(!$opts{o})||($opts{h}))
{
	print "
USAGE: $0 -g <genome-file> -d <training-dir-for-genome> -o <output> [options]
Options:
-g file_name     Genome file in fasta
-p file_name     If protein domain searches are available, read them from file file_name
-d trained_dir   Training directory is specified by dir_name
-w out_dir       Output dir, prediction results and temp files are placed here
-o out_prefix    Output prefix, prediction results are named this.
-n n             Print top n best predictions(default:1)
-gff             Print output in gff format
-v               Don't use svm splice site predictions
-f               Don't make partial gene predictions
-t num_threads   Number of Threads to run(default:1)
-h               Display the usage

NOTE:Please make sure glimmerhmm_linux is in your PATH
";
	exit(1);
}
$genome=rel2abs($opts{g});
$pds=$opts{p};
$traindir=rel2abs($opts{d});
$outpfx=$opts{o}?$opts{o}:basename($genome);
$wd=$opts{w}?rel2abs($opts{w}):'.';
$bestn=($opts{n})?$opts{n}:1;
$gff=$opts{gff};
$nosvm=$opts{v};
$nopartial=$opts{f};
$np=$opts{t}?$opts{t}:1;


#prepare work dir 	#ver 0.2
die "[error] $wd not a valid directory!" if ((-e $wd)&&(!-d $wd));
mkdir $wd if (!-e $wd);
system "ln -sf $genome $wd/".basename($genome);
$genome="$wd/".basename($genome);
system "rm -f $wd/$outpfx.ghmm.*";	#clean last run result

#split seq	#ver 0.2
&splitter($genome,$np);

#start glimmerhmm 
#create np threads
my %thread;
for $partnum(1..$np){
	my $thread=Thread->new(\&thread_wrapper,"$genome.$partnum",$partnum,$traindir,$pds,$wd,$outpfx,$bestn,$gff,$nosvm,$nopartial);
	$thread{$thread->tid()}=$thread;
	print "Thread $partnum created, tid=".$thread->tid()."\n";
}
#watch threads
my $running=1;
while ($running){
	$running=0;
	foreach $tid(keys %thread){
		my $thread=Thread->object($tid);
		if ($thread->is_running()){
			$running++;
		}else{
			$thread->join();
			delete $thread{$tid};
			print "Thread tid=$tid finished.\n";
		}
	}
	sleep 1;
}
#recycle
system "cat $wd/$outpfx.ghmm.* > $wd/$outpfx.ghmm";
print "********************FINISH AUTO GlimmerHMM********************\n";


#SUB:thread_wrapper	#ver 0.2(2012/2/10)
sub thread_wrapper{
	#threaded genscan wrapper
	#in: $genome,$partnum,$pds,$wd,$outpfx,$bestn,$gff,$nosvm,$nopartial;
	#ret: is_success?
	my ($genome,$partnum,$traindir,$pds,$wd,$outpfx,$bestn,$gff,$nosvm,$nopartial)=@_;
	my $all;
	my $err;
	my $seqio=Bio::SeqIO->new(-file=>$genome,-format=>'fasta');
	while (my $seq=$seqio->next_seq()){
		$all++;
		my $seqout=Bio::SeqIO->new(-file=>">$wd/seqtmp.$partnum.$all",-format=>'fasta');
		$seqout->write_seq($seq);
		$cmd="glimmerhmm_linux ";
		$cmd.="$wd/seqtmp.$partnum.$all ";
		$cmd.="$traindir ";
		$cmd.="-o $wd/$outpfx.ghmm.$partnum.$all ";
		$cmd.="-n $bestn ";
		$cmd.="-v " if ($nosvm);
		$cmd.="-f " if ($nopartial);
		$cmd.="-p $pds " if ($pds);
		$cmd.="-g " if ($gff);
		$err=system $cmd;
		if ($err){return $err;}
		if ($bestn == 1){
			system "cat $wd/$outpfx.ghmm.$partnum.$all >> $wd/$outpfx.ghmm.$partnum";
			system "rm -f $wd/$outpfx.ghmm.$partnum.$all";
		}
		else {
			system "cat $wd/$outpfx.ghmm.$partnum.$all.* >> $wd/$outpfx.ghmm.$partnum";
			system "rm -f $wd/$outpfx.ghmm.$partnum.$all.*";
		}
		system "rm -f $wd/seqtmp.$partnum.$all";
	}
	return 0;
}


#SUB:splitter #ver 0.2(2012/2/6)
sub splitter{
	#fasta seq splitter
	#in: full_path_fasta, part
	#out: no
	my $fasta=shift @_;
	my $part=shift @_;
	my $seqall=`grep -c \">\" $fasta`;
	my $partsize=int($seqall/$part)+1;
	my $seqcount;
	my $partnow;
	my $seqin=Bio::SeqIO->new(-file=>$fasta,-format=>'fasta');
	my $seqout;
	while (my $seq=$seqin->next_seq()){
		$seqcount++;
		if ($seqcount > $partsize*$partnow){
			$partnow++;
			$seqout=Bio::SeqIO->new(-file=>">$fasta.$partnow",-format=>'fasta');
		}
		$seqout->write_seq($seq);
	}
	return 1;
}