#!/usr/bin/perl -w
#***************************************
#program: gff3_filter.pl
#Description: this script remove genes contain N less than 30% 
#Author: Wang Xin <wang.xin@kuast.edu.sa>
#Date:   2015/5/25
#***************************************
use strict;
use Bio::Seq;
use Bio::SeqIO;
use File::Basename qw<basename dirname>;
use Getopt::Long;

my $version="V1.00";
my %opts;
GetOptions(\%opts,"i:s","f:s","o:s","p:s","h");
if (!defined $opts{i}||!defined$opts{f}||!defined$opts{o}||!defined$opts{p} || defined $opts{h})
 {
   print "
Program : $0
Version : $version
Contact : Wang xin <wang.xin\@kaust.edu.sa>
Usage :$0  <-i fasta file> <-f GFF file> <-o GFF file> -p percentage [option]
options:
	-i genome fasta file/Users/xinw/Desktop/remove_N_70.pl
	-f GFF input file
	-o GFF out file 
	-p N percentage of total gene
	-h help
";
	exit(1);		 
	
}

my $genome=$opts{i};
my $gff=$opts{f};
my $output=$opts{o};
my $percentage=$opts{p}?$opts{p}:0.7;


my %sequence;
my $seqio=Bio::SeqIO->new(-file=>$genome,-format=>'fasta');
while (my $seq=$seqio->next_seq()){
	my $id=$seq->id();
	my ($Nid)=$id=~/^(\S+)/;
	my $sequence=$seq->seq();
	$sequence{$Nid}=$sequence;
}


open GFF,"$gff" or die "Cannot open GFF file:$gff";
open OUT,">$output" or die "Cannot create GFF file: $output";
my %N;
while (<GFF>){
	chomp;
	if (/^#/){next;}
	my ($tid,$source,$type,$start,$end,$score,$dir,$phase,$attrib)=split(/\t/,$_);
	$attrib=(split /=|;/ ,$attrib)[1];
	my $gene_id = ($type=~/gene/)?$attrib:(split/\.t/,$attrib)[0];
	#print "$gene_id\n";
	if ($type =~ /gene/i){
		#$gene_id= $attrib=~ /ID=(\S+)/;
		my $gene=substr($sequence{$tid},$start,abs($end-$start)+1);
		my $length_gene= abs($end-$start)+1;
		#print "$length_gene\n";
		my $all_gene_N=0;
		if (my @N_cluter=$gene=~/N+/gi) {
			for my $N_num(@N_cluter) {
				#print "$N_num\n";
					$all_gene_N+=length $N_num;
				}
		}
		my $percent_N=$all_gene_N/$length_gene;
		#print "$percent_N\n";
		if ($all_gene_N/$length_gene < $percentage){
			print "$percent_N\n";
			$N{$gene_id}++;
			print OUT "$_\n";
		}
	}else{
		if ($N{$gene_id}){
			print OUT "$_\n";
		}
	}
#		while ($attrib =~ /ID=([^\s;]+)/){
#			my $temp = $1;
			#$gene_id = (split /\.t/, $temp)[0];
#			$temp =~s/(\S+)\.t*/$1/;
#			$gene_id=$temp;
		#	if ($N{$gene_id}){
		#		print OUT "$_\n";
		#	}
#	}
#	}
	
	
}

close OUT;
close GFF;